package com.winhc.bigdata.spark.jobs.chance

import com.winhc.bigdata.spark.utils.BaseUtil.cleanup
import com.winhc.bigdata.spark.utils.ChangeExtractUtils
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging

import scala.annotation.meta.{getter, setter}

/**
 * @Author: XuJiakai
 * @Date: 2020/7/9 16:44
 * @Description:
 */

trait CompanyChangeHandle extends Serializable with Logging {
  @getter
  @setter
  protected val equCols: Seq[String]

  /**
   *
   * @param rowkey
   * @param oldMap
   * @param newMap
   * @return rowkey,cid,类型【insert or update】,新数据，更新字段，更新标题，变更标签【1.一般变更，2.风险变更 ...】,业务时间
   */
  def handle(rowkey: String, oldMap: Map[String, String], newMap: Map[String, String]): (String, String, String, Map[String, String], String, String, String, String, Map[String, String]) = {
    if (oldMap == null) {
      (rowkey, getCid(rowkey, newMap), "insert", newMap, null, getInsertTitle(newMap), getLabel(oldMap, newMap), getBizTime(newMap), null)
    } else {
      val t = getEquAndFields(oldMap, newMap)
      if (t._1) {
        null
      } else {
        (rowkey, getCid(rowkey, newMap), "update", newMap,
          t._2
          , getUpdateTitle(newMap), getLabel(oldMap, newMap), getBizTime(newMap), oldMap)
      }
    }
  }

  def getCid(rowkey: String, newMap: Map[String, String]): String = rowkey.split("_")(0)

  def getUpdateTitle(newMap: Map[String, String]): String

  def getInsertTitle(newMap: Map[String, String]): String

  def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String

  def getBizTime(newMap: Map[String, String]): String

  def getEquAndFields(oldMap: Map[String, String], newMap: Map[String, String]): (Boolean, String) = {
    val tmp = equCols.map(f => {
      (f, cleanup(newMap(f)).equals(cleanup(oldMap(f))))
    })
    val eq = tmp.map(_._2).reduce((a1, a2) => a1 && a2)
    if (eq) {
      (true, null)
    } else {
      (eq, tmp.filter(!_._2).map(_._1).mkString(","))
    }
  }


  protected def getValueOrNull(value: String, callBack: String): String = {
    if (StringUtils.isNotBlank(value)) {
      callBack
    } else {
      null
    }
  }
}

//土地公示
case class company_land_publicity(equCols: Seq[String]) extends CompanyChangeHandle with Serializable {

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = {
    val str = ChangeExtractUtils.getTags(newMap, "地块公示", Array("project_name", "location", "application_name", "area", "final_price", "land_num"))
    str
  }

  override def getBizTime(newMap: Map[String, String]): String = newMap("publication_start_date")

  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("project_name"), s"${newMap("project_name")}地块公示发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("project_name"), s"新增${newMap("project_name")}地块公示")
}


case class company(equCols: Seq[String]) extends CompanyChangeHandle with Serializable {
  override def getCid(rowkey: String, newMap: Map[String, String]): String = rowkey

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = "1"

  override def getBizTime(newMap: Map[String, String]): String = "业务时间"

  override def getUpdateTitle(newMap: Map[String, String]): String = "更新一家公司"

  override def getInsertTitle(newMap: Map[String, String]): String = "新增一家公司"
}

//商标
case class company_tm(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("tm_name"), s"${newMap("tm_name")}商标发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("tm_name"), s"新增${newMap("tm_name")}商标")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.get_ip_tags("商标", newMap("tm_name"), newMap("app_date"), newMap("reg_no"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("app_date")
}

//专利
case class company_patent_list(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("title"), s"${newMap("title")}专利发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("title"), s"新增${newMap("title")}专利")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.get_ip_tags("专利", newMap("title"), newMap("app_date"), newMap("app_number"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("app_date")
}

//资质证书
case class company_certificate(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("cert_no"), s"${newMap("cert_no")}资质证书发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("cert_no"), s"新增${newMap("cert_no")}资质证书")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.get_ip_tags("资质证书", newMap("cert_no"), newMap("start_date"), newMap("type"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("start_date")
}

//作品著作权
case class company_copyright_works_list(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("name"), s"${newMap("name")}作品著作权发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("name"), s"新增${newMap("name")}作品著作权")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.get_ip_tags("作品著作权", newMap("name"), newMap("reg_time"), newMap("reg_num"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("reg_time")
}

//软件著作权
case class company_copyright_reg_list(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("full_name"), s"${newMap("full_name")}软件著作权发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("full_name"), s"新增${newMap("full_name")}软件著作权")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.get_ip_tags("软件著作权", newMap("full_name"), newMap("reg_time"), newMap("reg_num"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("reg_time")
}


//购地信息
case class company_land_announcement(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("project_name"), s"${newMap("project_name")}购地信息发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("project_name"), s"新增${newMap("project_name")}购地信息")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.getTags(newMap, "购地信息", Array("project_name", "project_loc", "area", "tran_price", "e_number"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("contract_date")
}

//招聘
case class company_employment(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("title"), s"${newMap("title")}招聘信息发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("title"), s"新增${newMap("title")}招聘信息")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.getTags(newMap, "招聘", Array("title", "city->employment_city", "employ_num", "start_date"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("start_date")
}

//招投标
case class company_bid_list(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("title"), s"${newMap("title")}招投标信息发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("title"), s"新增${newMap("title")}招投标信息")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.getTags(newMap, "招投标", Array("publish_time", "title", "purchaser", "province", "abs"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("publish_time")
}

//土地转让
case class company_land_transfer(equCols: Seq[String]) extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("location"), s"${newMap("title")}土地转让信息发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("location"), s"新增${newMap("location")}土地转让信息")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.getTags(newMap, "土地转让", Array("merchandise_time", "user_pre", "user_now", "location", "area", "merchandise_price", "aministrative_area"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("merchandise_time")
}
//环保处罚
case class company_env_punishment(equCols:Seq[String])extends CompanyChangeHandle {
  override def getUpdateTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("punish_number"), s"${newMap("title")}环保处罚信息发生变更")

  override def getInsertTitle(newMap: Map[String, String]): String = getValueOrNull(newMap("punish_number"), s"新增${newMap("punish_number")}环保处罚信息")

  override def getLabel(oldMap: Map[String, String], newMap: Map[String, String]): String = ChangeExtractUtils.getTags(newMap, "环保处罚", Array("name", "department", "punish_number", "punish_basis", "law_break", "reason", "content", "deleted"))

  override def getBizTime(newMap: Map[String, String]): String = newMap("publish_time")
}
